@@ -56,13 +56,13 @@ def parse_args(args):
         "-s",
         "--start",
         type=check_positive,
-        help="from which position in the playlist to start downloading",
+        help="from which position in the playlist to start downloading (defaults to 1)",
     )
     parser.add_argument(
         "-e",
         "--end",
         type=check_positive,
-        help="position in the playlist of the last song to be downloaded",
+        help="position in the playlist of the last song to be downloaded (defaults to last position in the playlist)",
     )
     parser.add_argument(
         "-A",
@@ -110,8 +110,6 @@ def parse_args(args):
 
 def main():
     if "--check" in sys.argv[1:] or "-k" in sys.argv[1:]:
-        print("Initialising.")
-        colorama.init()
         urls = Playlist("https://www.youtube.com/playlist?list=PLOoPqX_q5JAVPMhHjYxcUc2bxTDMyGE-a")
         playlist_title = urls.title()
         start = 0
@@ -126,8 +124,6 @@ def main():
         mp3 = True
 
     else:
-        print("Initialising.")
-        colorama.init()
         args = parse_args(sys.argv[1:])
         urls = Playlist(args.url)
         playlist_title = urls.title()
@@ -150,6 +146,8 @@ def main():
                 proxies[p[0]] = p[1]
 
 
+    print("Initialising.")
+    colorama.init()
     try:
         if start >= len(urls):
             raise error.InvalidPlaylistIndexError(start, playlist_title)
