@@ -1,10 +1,11 @@
-import requests
 import os 
+import requests
+import asyncio
 
 from pytube import YouTube, Playlist
 from mutagen.mp4 import MP4, MP4Cover
-from mutagen.id3 import ID3, TPE1, TIT2, TRCK, TALB, APIC
-import moviepy.editor as mp
+
+from ffmpeg import FFmpeg
 
 try:
     import font
@@ -29,6 +30,10 @@ def extract_title(string):
 def extract_ext(string):
     return string.split(".")[1]
 
+def to_sec(timestamp):
+    sep = timestamp.split(":")
+    h, m, s = (int(sep[0]), int(sep[1]), float(sep[2]))
+    return (h * 60 * 60) + (m * 60) + round(s)
 
 class Downloader:
     is_album = None
@@ -72,33 +77,21 @@ class Downloader:
 
         print(progress, end=end, flush=True)
 
-    def apply_metadata(
-        self, track_num, total, path, album, title, artist, image_filename, mp3
-    ):
-        if mp3:
-            song =  ID3(path)
-            song['TPE1'] = TPE1(encoding=3, text=artist)
-            song['TIT2'] = TIT2(encoding=3, text=title)
-            song['TRCK'] = TRCK(encoding=3, text=str(track_num))
-            song['TALB'] = TALB(encoding=3, text=album)
-            with open(image_filename, 'rb') as albumart:
-                song['APIC'] = APIC(
-                                  encoding=3,
-                                  mime=f"image/{extract_ext(image_filename)}",
-                                  type=3, desc=u'Cover',
-                                  data=albumart.read()
-                                )
-            song.save()
-        else:
-            song = MP4(path)
-            song["\xa9alb"] = album
-            song["\xa9nam"] = title
-            song["\xa9ART"] = artist
-            song["trkn"] = [(track_num, total)]
 
+    def apply_metadata(
+        self, track_num, total, path, album, title, artist, image_filename
+    ):
+        song = MP4(path)
+        song["\xa9alb"] = album
+        song["\xa9nam"] = title
+        song["\xa9ART"] = artist
+        song["trkn"] = [(track_num, total)]
+
+        if image_filename is not None:
             with open(image_filename, "rb") as f:
                 song["covr"] = [MP4Cover(f.read(), imageformat=MP4Cover.FORMAT_JPEG)]
-            song.save()
+        
+        song.save()
         
 
     @staticmethod
@@ -116,6 +109,7 @@ class Downloader:
             tg.make_titles()
             metadata = tg.get_titles()
 
+        loop = asyncio.get_event_loop()
         for num, url in self.urls:
             yt = None
             self.cur_song = num+self.start+1
@@ -126,7 +120,7 @@ class Downloader:
                     yt = YouTube(url)
 
             except Exception as e:
-                self.retry_urls.append(url)
+                self.retry_urls.append((num, url))
                 print(
                     f"Downloading song {font.apply('gb', str(self.cur_song))} - {font.apply('bf', '[Failed - ')} {font.apply('bf', str(e) + ']')}\n"
                 )
@@ -135,18 +129,12 @@ class Downloader:
             path = None
             try:
                 yt.register_on_progress_callback(self.progress_function)
-                if self.mp3:
-                    self.cur_video = (
-                        yt.streams.filter(res="360p")
-                        .first()
-                    )
-                else:
-                    self.cur_video = (
-                        yt.streams.filter(type="audio", subtype="mp4")
-                        .order_by("abr")
-                        .desc()
-                        .first()
-                    )
+                self.cur_video = (
+                    yt.streams.filter(type="audio", subtype="mp4")
+                    .order_by("abr")
+                    .desc()
+                    .first()
+                )
                 
                 safe_name = extract_title(make_safe_filename(self.cur_video.title))
                 path = self.cur_video.download(
@@ -163,19 +151,19 @@ class Downloader:
 
                 continue
 
-            if self.is_album:
-                if self.image_filepath is None:
-                    if not self.album_image_set:
-                        image_path = Downloader.download_image(
-                            yt.thumbnail_url, num, self.outdir
-                        )
-                        self.images.append(image_path)
-                        self.image_filepath = image_path
-                        self.album_image_set = True
-            else:
-                image_path = Downloader.download_image(yt.thumbnail_url, num, self.outdir)
-                self.images.append(image_path)
-                self.image_filepath = image_path
+            # if self.is_album:
+            #     if self.image_filepath is None:
+            #         if not self.album_image_set:
+            #             image_path = Downloader.download_image(
+            #                 yt.thumbnail_url, num, self.outdir
+            #             )
+            #             self.images.append(image_path)
+            #             self.image_filepath = image_path
+            #             self.album_image_set = True
+            # else:
+            #     image_path = Downloader.download_image(yt.thumbnail_url, num, self.outdir)
+            #     self.images.append(image_path)
+            #     self.image_filepath = image_path
 
             track_title = None
             track_artist = None
@@ -187,16 +175,7 @@ class Downloader:
                 track_title = self.cur_video.title
                 track_artist = self.artist
 
-            if self.mp3:
-                try:
-                    print(f"└── Converting to mp3", end="\r")
-                    clip = mp.VideoFileClip(path)
-                    path = f"{extract_title(path)}.mp3"
-                    clip.audio.write_audiofile(path, verbose=False, logger=None)
-                    print(f"├── Converting to mp3 - {font.apply('bl', '[Done]')}")
-                    os.remove(f"{extract_title(path)}.mp4")
-                except Exception as e:
-                    print(f"├── Converting to mp3 - {font.apply('bf', '[Failed - ')} {font.apply('bf', str(e) + ']')}")
+            metadata_branch = "├──" if self.mp3 else "└──"
 
             try:
                 self.apply_metadata(
@@ -206,17 +185,39 @@ class Downloader:
                     self.album,
                     track_title,
                     track_artist,
-                    self.image_filepath,
-                    self.mp3
+                    self.image_filepath
                 )
-                print(f"└── Applying metadata - {font.apply('bl', '[Done]')}")
+                print(f"{metadata_branch} Applying metadata - {font.apply('bl', '[Done]')}")
 
             except Exception as e:
-                print(f"└── Applying metadata - {font.apply('bf', '[Failed - ')} {font.apply('bf', str(e) + ']')}")
+                print(f"{metadata_branch} Applying metadata - {font.apply('bf', '[Failed - ')} {font.apply('bf', str(e) + ']')}")
+            
+            if self.mp3:
+                ffmpeg = FFmpeg().input(path).output(f"{extract_title(path)}.mp3")
 
+                @ffmpeg.on('progress')
+                def mp3_conv_progress(event):
+                    p = (to_sec(event.time)/int(yt.length)) * 100 
+                    progress = (
+                        f"└── Converting to mp3 - [{p:.2f}%]"
+                        if p < 100
+                        else f"└── Converting to mp3 - {font.apply('bl', '[Done]          ')}"
+                    )
 
+                    end = "\n" if p >= 100 else "\r"
+                    print(progress, end=end, flush=True)
+
+                try:
+                    loop.run_until_complete(ffmpeg.execute())
+
+                    os.remove(f"{extract_title(path)}.mp4")
+                    path = f"{extract_title(path)}.mp3"
+                
+                except Exception as e:
+                    print(f"└── Converting to mp3 - {font.apply('bf', '[Failed - ')} {font.apply('bf', str(e) + ']')}")
 
             print(" ")
+        loop.close()
 
 
 
